package hu.bme.mit.inf.hidenets.eval.plugin.carwfimporter;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.imports.NativeImporter;

public class CarWFImporter {
	IModelManager mm = null;  
	IEntity vModel = null;
	String META_carworkflow = "carworkflow.metamodels.carWorkflow_metamodel.carworkflow"; 	
	
	/**
	 * 
	 * @param f The InputStream of the file being imported. 
	 * @param fw The VIATRA2 framework.
	 * @param vModelsFQN The fully qualified name of the entity containing the carworkflow models. 
	 * @param modelName The name of the carworkflow model being imported. 
	 * @return The new carworkflow model is being returned. 
	 * @throws VPMRuntimeException
	 */
	public IEntity process(InputStream f, IFramework fw,String vModelsFQN,String modelName)
	throws VPMRuntimeException {
		try {
			//Initing the DOM parser. 
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder;
			builder = factory.newDocumentBuilder();
			Document doc;
			doc = builder.parse(f);
			NodeList docsChildren = doc.getChildNodes();
			
			//Searching the XML node of the carworkflow. This is the top node of the tree. 
			Node xCarWorkflow = null;
			for(int i = 0;i<docsChildren.getLength();i++){
				if(docsChildren.item(i).getNodeType()==Node.ELEMENT_NODE){
					xCarWorkflow = docsChildren.item(i);
				}
			}
			
			//Instantiating the root of the model. 
			mm = fw.getTopmodel().getModelManager();
			IEntity VModels = mm.getEntityByName(vModelsFQN);
			vModel = mm.newEntity(modelName,VModels);
			IEntity vMetaCarWorkflow = mm.getEntityByName(META_carworkflow);
			IEntity vCarWorkflow  = mm.newEntity(vModel);
			mm.newInstanceOf(vMetaCarWorkflow,vCarWorkflow);
			
			//Calling the core of the importer, the DFN. 
			parsenode(xCarWorkflow,vCarWorkflow);	
			return vModel;
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			return null;
		} catch (SAXException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		} catch (VPMCoreException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * This recursive method is the core of the importer. This builds the containment tree in the VIATRA2 workspace based on the input.
	 * The interesting thing is that it that this code fragment is independent of the metamodel. The metamodel elements are found based on 
	 * the names of the XML tags. 
	 * @param xSource An XML tag. Here, a container.
	 * @param vSource The VIATRA2 instance of the same container node. 
	 * @throws VPMCoreException
	 */
	private void parsenode(Node xSource,IEntity vSource) throws VPMCoreException {
		ArrayList<IModelElement> vMetaSet = new ArrayList<IModelElement>(vSource.getAllTypes());

		for (IModelElement vMetaSource:vMetaSet) {
			NodeList xChildren = xSource.getChildNodes();
			for(int i=0;i<xChildren.getLength();i++){
				//A child of xSource
				Node xTarget = xChildren.item(i);
				if (xTarget.getNodeType()==Node.ELEMENT_NODE) {
					String xTargetName = xTarget.getNodeName();
					//The containment relation found by the tag name. 
					IRelation vMetaRelation = vMetaSource.getRelationFromByName(xTargetName);
					IEntity vMetaTarget = (IEntity) vMetaRelation.getTo();
					/*
					 * This "if" presumes that there are no multiple metarelations with the same
					 * name among the metaobjects.   
					 */
					if (vMetaRelation != null) {
						IEntity vTarget = mm.newEntity("_" + xTargetName,vModel);
						mm.newInstanceOf(vMetaTarget, vTarget);

						IRelation vRelation = mm.newRelation(xTargetName,vSource, vTarget);
						mm.newInstanceOf(vMetaRelation, vRelation);

						parsenode(xTarget, vTarget);
					}
				}
			}
		}
	}
	
	/**
	 * This method imports the carworkflow in the specified file.
	 * @param f The file name in the file system.
	 * @param fw The VIATRA2 framework.
	 * @param vModelsFQN The fully qualified name of the VIATRA2 entity containing the models.
	 * @param modelName The name of the model being imported. This is going to be the name of the VIATRA2 entity containing the model instance.
	 * @return	The container (model) entity of the VIATRA2 model is returned. 
	 * @throws VPMRuntimeException
	 */
	public IEntity processFile(String f, IFramework fw,String vModelsFQN,String modelName) throws VPMRuntimeException {
		try {
			FileInputStream fileInputStream = new FileInputStream(f);
			File file = new File(f);
			return process(fileInputStream,fw,vModelsFQN,modelName);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
}
